require "cromwell_api"

module CromwellApi
module Rack
  class RequestTime

    attr_reader :config
    def initialize(app, config)
      @app = app
      @config = config
    end

    def call(env)
      start_time = Time.now
      response = @app.call(env)
      request_time = ((Time.now - start_time) * 1000).round

      if response[0] == 200
        client.send_metric metric_key(env), request_time
      end

      response
    end

    def client
      Thread.current[:cromwell_client] ||= begin
        options = {}
        non_option_keys = [:product, :host]
        config.each do |key, value|
          key = key.to_sym
          unless non_option_keys.include? key
            options[key] = value
          end
        end
        ::CromwellApi::Client.new config[:product], env, config[:host], options
      end
    end

    def env
      ENV.fetch "RACK_ENV", "production"
    end

    def metric_key(env)
      # Requires sinatra 1.4+
      if env["sinatra.route"]
        method, route = env["sinatra.route"].split
        route
      else
        "request_time"
      end
    end
  end

end
end
